<?php

$install_path = 'add_installation_path_here';

include($install_path . "/parameters.inc.php");

$bmaxup       = $highspeed_contracts_min_upload;
$bmaxdown     = $highspeed_contracts_min_download;
$dbu          = 'add_database_username_here';
$dbpsw        = 'add_database_password_here';
$database     = 'add_database_name_here';

include($install_path . "/library.lib.php");

$spebs_db = connectDB();

$queries = array();
$queries["sliding window       "] = "SET @intv = $sliding_window_in_days";
$queries["min measurement count"] = "SET @minmeasurements = $min_measurements_per_user";

/**********   View containing aggregates per user       **************/

// All combinations of workingDay X peakHour
$queries["connection: wD, wH   "] = "CREATE TABLE aggregation_per_connection_new AS 
(SELECT m.connection_id, m.isp_id, m.workingDay, m.peakHour, avg(upstream_bw) avgup, avg(downstream_bw) avgdown, avg(rtt) avgrtt, avg(loss) avgloss, avg(jitter) avgjitter,  IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM generic_measurements_stats m 
JOIN connection c ON m.connection_id=c.connection_id 
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, workingDay, peakHour)";

// All time periods for every workingDay 
$queries["connection: wD       "] = "INSERT INTO aggregation_per_connection_new
(SELECT m.connection_id, m.isp_id, workingDay, -1, avg(upstream_bw) avgup, avg(downstream_bw) avgdown, avg(rtt) avgrtt, avg(loss) avgloss, avg(jitter) avgjitter, IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM generic_measurements_stats  m 
JOIN connection c ON m.connection_id=c.connection_id 
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, workingDay)";

// All days for every time period
$queries["connection: wH       "] = "INSERT INTO aggregation_per_connection_new 
(SELECT m.connection_id, m.isp_id, -1, peakHour, avg(upstream_bw) avgup, avg(downstream_bw) avgdown, avg(rtt) avgrtt, avg(loss) avgloss, avg(jitter) avgjitter, IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM generic_measurements_stats  m 
JOIN connection c ON m.connection_id=c.connection_id 
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, peakHour)";

// Everything grouped per user
$queries["connection: all      "] = "INSERT INTO aggregation_per_connection_new 
(SELECT m.connection_id, m.isp_id, -1, -1, avg(upstream_bw) avgup, avg(downstream_bw) avgdown, avg(rtt) avgrtt, avg(loss) avgloss, avg(jitter) avgjitter, IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM generic_measurements_stats m 
JOIN connection c ON m.connection_id=c.connection_id 
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id)";

$queries["connection: drop     "] = "DROP TABLE IF EXISTS aggregation_per_connection"; 
$queries["connection: rename   "] = "RENAME TABLE aggregation_per_connection_new TO aggregation_per_connection"; 
$queries["connection: index    "] = "CREATE INDEX connindex ON aggregation_per_connection(connection_id)"; 
$queries["connection: analyze  "] = "ANALYZE TABLE aggregation_per_connection"; 


/**********   View containing aggregates per TK     **************/

//Everything grouped per postal_code for all contracts
$queries["postal code: all     "] = "CREATE TABLE aggregation_per_postal_code_new AS 
(SELECT postal_code, isp_id, workingDay, peakHour, -1 contract, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements
GROUP BY postal_code, isp_id, workingDay, peakHour)";

//Everything grouped per postal_code for highest speed contract
$queries["postal code: high    "] = "INSERT INTO aggregation_per_postal_code_new 
(SELECT postal_code, isp_id, workingDay, peakHour, 0, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY postal_code, isp_id, workingDay, peakHour)";

$queries["postal code: drop    "] = "DROP TABLE IF EXISTS aggregation_per_postal_code"; 
$queries["postal code: rename  "] = "RENAME TABLE aggregation_per_postal_code_new TO aggregation_per_postal_code"; 
$queries["postal code: index   "] = "CREATE INDEX pcindex ON aggregation_per_postal_code(postal_code, workingDay, peakHour, contract)"; 
$queries["postal code: analyze "] = "ANALYZE TABLE aggregation_per_postal_code"; 


/**********   View containing aggregates per Municipality     **************/

//Everything grouped per municipality for all contracts
$queries["municipality: all    "] = "CREATE TABLE aggregation_per_municipality_new AS 
(SELECT municipality, isp_id, workingDay, peakHour, -1 contract, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements
GROUP BY municipality, isp_id, workingDay, peakHour)";

//Everything grouped per municipality for highest  speed contract
$queries["municipality: high   "] = "INSERT INTO aggregation_per_municipality_new 
(SELECT municipality, isp_id, workingDay, peakHour, 0, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY municipality, isp_id, workingDay, peakHour)";

$queries["municipality: drop   "] = "DROP TABLE IF EXISTS aggregation_per_municipality"; 
$queries["municipality: rename "] = "RENAME TABLE aggregation_per_municipality_new TO aggregation_per_municipality"; 
$queries["municipality: index  "] = "CREATE INDEX mindex ON aggregation_per_municipality(municipality, workingDay, peakHour, contract)"; 
$queries["municipality: analyze"] = "ANALYZE TABLE aggregation_per_municipality"; 


/**********   View containing aggregates per Prefecture     **************/

//Everything grouped per prefecture for all contracts
$queries["prefecture: all      "] = "CREATE TABLE aggregation_per_prefecture_new AS 
(SELECT prefecture, isp_id, workingDay, peakHour, -1 contract, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements
GROUP BY prefecture, isp_id, workingDay, peakHour)";


//Everything grouped per prefecture for highest speed contract
$queries["prefecture: high     "] = "INSERT INTO aggregation_per_prefecture_new 
(SELECT prefecture, isp_id, workingDay, peakHour, 0, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY prefecture, isp_id, workingDay, peakHour)";

$queries["prefecture: drop     "] = "DROP TABLE IF EXISTS aggregation_per_prefecture"; 
$queries["prefecture: rename   "] = "RENAME TABLE aggregation_per_prefecture_new TO aggregation_per_prefecture"; 
$queries["prefecture: index    "] = "CREATE INDEX pindex ON aggregation_per_prefecture(prefecture, workingDay, peakHour, contract)"; 
$queries["prefecture: analyze  "] = "ANALYZE TABLE aggregation_per_prefecture"; 


/**********   View containing aggregates per Periphery     **************/

//Everything grouped per periphery for all contracts
$queries["periphery: all       "] = "CREATE TABLE aggregation_per_periphery_new AS 
(SELECT periphery, isp_id, workingDay, peakHour, -1 contract, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements
GROUP BY periphery, isp_id, workingDay, peakHour)";

//Everything grouped per periphery for highest speed contract
$queries["periphery: high      "] = "INSERT INTO aggregation_per_periphery_new 
(SELECT periphery, isp_id, workingDay, peakHour, 0, avg(avgup) avgup, avg(avgdown) avgdown, avg(avgrtt) avgrtt, avg(avgloss) avgloss, avg(avgjitter) avgjitter, count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY periphery, isp_id, workingDay, peakHour)";

$queries["periphery: drop      "] = "DROP TABLE IF EXISTS aggregation_per_periphery"; 
$queries["periphery: rename    "] = "RENAME TABLE aggregation_per_periphery_new TO aggregation_per_periphery"; 
$queries["periphery: index     "] = "CREATE INDEX pindex ON aggregation_per_periphery(periphery, workingDay, peakHour, contract)"; 
$queries["periphery: analyze   "] = "ANALYZE TABLE aggregation_per_periphery"; 


/*=============================================================    Glasnost    ======================================================================================*/
$queries["min measurement count_g"] = "SET @minmeasurements = $min_glasnostmeasurements_per_user";
$queries["throttled      limit   "] = "SET @throttle_ratio = $glasnost_throttles_accepted_percentage";

/**********   View containing aggregates per user       **************/

// All combinations of workingDay X peakHour
$queries["connection_g: wD, wH   "] = "CREATE TABLE aggregation_per_connection_new AS 
(SELECT m.connection_id, m.isp_id, m.workingDay, m.peakHour, 
SUM(IF(protocol1 = 'BitTorrent', (download_indication OR upload_indication),0))  AS bittorent_throttled_measurements, 
SUM(IF(protocol1 = 'eMule', (download_indication OR upload_indication), 0))  AS emule_throttled_measurements, 
SUM(IF(protocol1 = 'Gnutella', (download_indication OR upload_indication), 0))  AS gnutella_throttled_measurements, 
SUM(IF(protocol1 = 'HTTP', (download_indication OR upload_indication), 0))  AS http_throttled_measurements,
SUM(IF(protocol1 = 'SSHTransfer', (download_indication OR upload_indication), 0))  AS ssh_throttled_measurements,
SUM(IF(protocol1 = 'POP', (download_indication OR upload_indication), 0))  AS pop_throttled_measurements,
SUM(IF(protocol1 = 'IMAP', (download_indication OR upload_indication), 0)) AS imap_throttled_measurements,
SUM(IF(protocol1 = 'FlashVideo', (download_indication OR upload_indication), 0))  AS flash_throttled_measurements,
COUNT(IF(protocol1 = 'BitTorrent', mid , NULL)) AS bittorent_measurements,
COUNT(IF(protocol1 = 'eMule', mid, NULL))  AS emule_measurements,
COUNT(IF(protocol1 = 'Gnutella', mid, NULL))  AS gnutella_measurements,
COUNT(IF(protocol1 = 'HTTP', mid, NULL))  AS http_measurements,
COUNT(IF(protocol1 = 'SSHTransfer', mid, NULL))  AS ssh_measurements,
COUNT(IF(protocol1 = 'POP', mid, NULL))  AS pop_measurements,
COUNT(IF(protocol1 = 'IMAP', mid, NULL))  AS imap_measurements,
COUNT(IF(protocol1 = 'FlashVideo', mid, NULL))  AS flash_measurements,
IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM glasnost_measurements_stats m JOIN connection c ON m.connection_id=c.connection_id
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, workingDay, peakHour)";

// All time periods for every workingDay 
$queries["connection_g: wD       "] = "INSERT INTO aggregation_per_connection_new
(SELECT m.connection_id, m.isp_id, workingDay, -1, 
SUM(IF(protocol1 = 'BitTorrent', (download_indication OR upload_indication), 0))  AS bittorent_throttled_measurements, 
SUM(IF(protocol1 = 'eMule', (download_indication OR upload_indication), 0))  AS emule_throttled_measurements, 
SUM(IF(protocol1 = 'Gnutella', (download_indication OR upload_indication), 0))  AS gnutella_throttled_measurements, 
SUM(IF(protocol1 = 'HTTP', (download_indication OR upload_indication), 0))  AS http_throttled_measurements,
SUM(IF(protocol1 = 'SSHTransfer', (download_indication OR upload_indication), 0))  AS ssh_throttled_measurements,
SUM(IF(protocol1 = 'POP', (download_indication OR upload_indication), 0))  AS pop_throttled_measurements,
SUM(IF(protocol1 = 'IMAP', (download_indication OR upload_indication), 0)) AS imap_throttled_measurements,
SUM(IF(protocol1 = 'FlashVideo', (download_indication OR upload_indication), 0))  AS flash_throttled_measurements,
COUNT(IF(protocol1 = 'BitTorrent', mid , NULL)) AS bittorent_measurements,
COUNT(IF(protocol1 = 'eMule', mid, NULL))  AS emule_measurements,
COUNT(IF(protocol1 = 'Gnutella', mid, NULL))  AS gnutella_measurements,
COUNT(IF(protocol1 = 'HTTP', mid, NULL))  AS http_measurements,
COUNT(IF(protocol1 = 'SSHTransfer', mid, NULL))  AS ssh_measurements,
COUNT(IF(protocol1 = 'POP', mid, NULL))  AS pop_measurements,
COUNT(IF(protocol1 = 'IMAP', mid, NULL))  AS imap_measurements,
COUNT(IF(protocol1 = 'FlashVideo', mid, NULL))  AS flash_measurements,
IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM glasnost_measurements_stats m JOIN connection c ON m.connection_id=c.connection_id
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, workingDay)";

// All days for every time period
$queries["connection_g: wH       "] = "INSERT INTO aggregation_per_connection_new 
(SELECT m.connection_id, m.isp_id, -1, peakHour, 
SUM(IF(protocol1 = 'BitTorrent', (download_indication OR upload_indication), 0))  AS bittorent_throttled_measurements, 
SUM(IF(protocol1 = 'eMule', (download_indication OR upload_indication), 0))  AS emule_throttled_measurements, 
SUM(IF(protocol1 = 'Gnutella', (download_indication OR upload_indication), 0))  AS gnutella_throttled_measurements, 
SUM(IF(protocol1 = 'HTTP', (download_indication OR upload_indication), 0))  AS http_throttled_measurements,
SUM(IF(protocol1 = 'SSHTransfer', (download_indication OR upload_indication), 0))  AS ssh_throttled_measurements,
SUM(IF(protocol1 = 'POP', (download_indication OR upload_indication), 0))  AS pop_throttled_measurements,
SUM(IF(protocol1 = 'IMAP', (download_indication OR upload_indication), 0)) AS imap_throttled_measurements,
SUM(IF(protocol1 = 'FlashVideo', (download_indication OR upload_indication), 0))  AS flash_throttled_measurements,
COUNT(IF(protocol1 = 'BitTorrent', mid , NULL)) AS bittorent_measurements,
COUNT(IF(protocol1 = 'eMule', mid, NULL))  AS emule_measurements,
COUNT(IF(protocol1 = 'Gnutella', mid, NULL))  AS gnutella_measurements,
COUNT(IF(protocol1 = 'HTTP', mid, NULL))  AS http_measurements,
COUNT(IF(protocol1 = 'SSHTransfer', mid, NULL))  AS ssh_measurements,
COUNT(IF(protocol1 = 'POP', mid, NULL))  AS pop_measurements,
COUNT(IF(protocol1 = 'IMAP', mid, NULL))  AS imap_measurements,
COUNT(IF(protocol1 = 'FlashVideo', mid, NULL))  AS flash_measurements,
IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM glasnost_measurements_stats m JOIN connection c ON m.connection_id=c.connection_id
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id, peakHour)";

// Everything grouped per user
$queries["connection_g: all      "] = "INSERT INTO aggregation_per_connection_new 
(SELECT m.connection_id, m.isp_id, -1, -1,
SUM(IF(protocol1 = 'BitTorrent', (download_indication OR upload_indication), 0))  AS bittorent_throttled_measurements, 
SUM(IF(protocol1 = 'eMule', (download_indication OR upload_indication), 0))  AS emule_throttled_measurements, 
SUM(IF(protocol1 = 'Gnutella', (download_indication OR upload_indication), 0))  AS gnutella_throttled_measurements, 
SUM(IF(protocol1 = 'HTTP', (download_indication OR upload_indication), 0))  AS http_throttled_measurements,
SUM(IF(protocol1 = 'SSHTransfer', (download_indication OR upload_indication), 0))  AS ssh_throttled_measurements,
SUM(IF(protocol1 = 'POP', (download_indication OR upload_indication), 0))  AS pop_throttled_measurements,
SUM(IF(protocol1 = 'IMAP', (download_indication OR upload_indication), 0)) AS imap_throttled_measurements,
SUM(IF(protocol1 = 'FlashVideo', (download_indication OR upload_indication), 0))  AS flash_throttled_measurements,
COUNT(IF(protocol1 = 'BitTorrent', mid , NULL)) AS bittorent_measurements,
COUNT(IF(protocol1 = 'eMule', mid, NULL))  AS emule_measurements,
COUNT(IF(protocol1 = 'Gnutella', mid, NULL))  AS gnutella_measurements,
COUNT(IF(protocol1 = 'HTTP', mid, NULL))  AS http_measurements,
COUNT(IF(protocol1 = 'SSHTransfer', mid, NULL))  AS ssh_measurements,
COUNT(IF(protocol1 = 'POP', mid, NULL))  AS pop_measurements,
COUNT(IF(protocol1 = 'IMAP', mid, NULL))  AS imap_measurements,
COUNT(IF(protocol1 = 'FlashVideo', mid, NULL))  AS flash_measurements,
IF(purchased_bandwidth_ul_kbps>=$bmaxup AND purchased_bandwidth_dl_kbps>=$bmaxdown,'high','low') contract_category, m.postal_code, m.municipality, m.prefecture, m.periphery, m.country, count(mid) measurements_count 
FROM glasnost_measurements_stats m JOIN connection c ON m.connection_id=c.connection_id
WHERE date_created > DATE_SUB(CURDATE(), INTERVAL @intv DAY) 
GROUP BY m.connection_id)";

$queries["connection_g: drop     "] = "DROP TABLE IF EXISTS aggregation_per_connection_glasnost"; 
$queries["connection_g: rename   "] = "RENAME TABLE aggregation_per_connection_new TO aggregation_per_connection_glasnost"; 
$queries["connection_g: index    "] = "CREATE INDEX connindex ON aggregation_per_connection_glasnost(connection_id)"; 
$queries["connection_g: analyze  "] = "ANALYZE TABLE aggregation_per_connection_glasnost"; 


/**********   View containing aggregates per TK     **************/

//Everything grouped per postal_code for all contracts
$queries["postal code_g: all     "] = "CREATE TABLE aggregation_per_postal_code_new AS 
(SELECT postal_code, isp_id,  workingDay, peakHour, -1 contract, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements
GROUP BY postal_code, isp_id, workingDay, peakHour)";

//Everything grouped per postal_code for highest speed contract
$queries["postal code_g: high    "] = "INSERT INTO aggregation_per_postal_code_new 
(SELECT postal_code, isp_id, workingDay, peakHour, 0, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY postal_code, isp_id, workingDay, peakHour)";

$queries["postal code_g: drop    "] = "DROP TABLE IF EXISTS aggregation_per_postal_code_glasnost"; 
$queries["postal code_g: rename  "] = "RENAME TABLE aggregation_per_postal_code_new TO aggregation_per_postal_code_glasnost"; 
$queries["postal code_g: index   "] = "CREATE INDEX pcindex ON aggregation_per_postal_code_glasnost(postal_code, workingDay, peakHour, contract)"; 
$queries["postal code_g: analyze "] = "ANALYZE TABLE aggregation_per_postal_code_glasnost"; 


/**********   View containing aggregates per Municipality     **************/

//Everything grouped per municipality for all contracts
$queries["municipality_g: all    "] = "CREATE TABLE aggregation_per_municipality_new AS 
(SELECT municipality, isp_id, workingDay, peakHour, -1 contract, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements
GROUP BY municipality, isp_id, workingDay, peakHour)";

//Everything grouped per municipality for highest  speed contract
$queries["municipality_g: high   "] = "INSERT INTO aggregation_per_municipality_new 
(SELECT municipality, isp_id, workingDay, peakHour, 0, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY municipality, isp_id, workingDay, peakHour)";

$queries["municipality_g: drop   "] = "DROP TABLE IF EXISTS aggregation_per_municipality_glasnost"; 
$queries["municipality_g: rename "] = "RENAME TABLE aggregation_per_municipality_new TO aggregation_per_municipality_glasnost"; 
$queries["municipality_g: index  "] = "CREATE INDEX mindex ON aggregation_per_municipality_glasnost(municipality, workingDay, peakHour, contract)"; 
$queries["municipality_g: analyze"] = "ANALYZE TABLE aggregation_per_municipality_glasnost"; 


/**********   View containing aggregates per Prefecture     **************/

//Everything grouped per prefecture for all contracts
$queries["prefecture_g: all      "] = "CREATE TABLE aggregation_per_prefecture_new AS 
(SELECT prefecture, isp_id, workingDay, peakHour, -1 contract, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements
GROUP BY prefecture, isp_id, workingDay, peakHour)";


//Everything grouped per prefecture for highest speed contract
$queries["prefecture_g: high     "] = "INSERT INTO aggregation_per_prefecture_new 
(SELECT prefecture, isp_id, workingDay, peakHour, 0, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY prefecture, isp_id, workingDay, peakHour)";

$queries["prefecture_g: drop     "] = "DROP TABLE IF EXISTS aggregation_per_prefecture_glasnost"; 
$queries["prefecture_g: rename   "] = "RENAME TABLE aggregation_per_prefecture_new TO aggregation_per_prefecture_glasnost"; 
$queries["prefecture_g: index    "] = "CREATE INDEX pindex ON aggregation_per_prefecture_glasnost(prefecture, workingDay, peakHour, contract)"; 
$queries["prefecture_g: analyze  "] = "ANALYZE TABLE aggregation_per_prefecture_glasnost"; 


/**********   View containing aggregates per Periphery     **************/

//Everything grouped per periphery for all contracts
$queries["periphery_g: all       "] = "CREATE TABLE aggregation_per_periphery_new AS 
(SELECT periphery, isp_id, workingDay, peakHour, -1 contract, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements
GROUP BY periphery, isp_id, workingDay, peakHour)";

//Everything grouped per periphery for highest speed contract
$queries["periphery_g: high      "] = "INSERT INTO aggregation_per_periphery_new 
(SELECT periphery, isp_id, workingDay, peakHour, 0, 
SUM(bittorent_measurements*@throttle_ratio<bittorent_throttled_measurements) bittorent_throttled_connections, SUM(bittorent_measurements) bittorent_measurements, SUM(bittorent_measurements>0) connections_with_bittorent_measurements,
SUM(emule_measurements*@throttle_ratio<emule_throttled_measurements) emule_throttled_connections, SUM(emule_measurements) emule_measurements, SUM(emule_measurements>0) connections_with_emule_measurements,
SUM(gnutella_measurements*@throttle_ratio<gnutella_throttled_measurements) gnutella_throttled_connections, SUM(gnutella_measurements) gnutella_measurements, SUM(gnutella_measurements>0) connections_with_gnutella_measurements,
SUM(http_measurements*@throttle_ratio<http_throttled_measurements) http_throttled_connections, SUM(http_measurements) http_measurements, SUM(http_measurements>0) connections_with_http_measurements,
SUM(ssh_measurements*@throttle_ratio<ssh_throttled_measurements) ssh_throttled_connections, SUM(ssh_measurements) ssh_measurements, SUM(ssh_measurements>0) connections_with_ssh_measurements,
SUM(pop_measurements*@throttle_ratio<pop_throttled_measurements) pop_throttled_connections, SUM(pop_measurements) pop_measurements, SUM(pop_measurements>0) connections_with_pop_measurements,
SUM(imap_measurements*@throttle_ratio<imap_throttled_measurements) imap_throttled_connections, SUM(imap_measurements) imap_measurements, SUM(imap_measurements>0) connections_with_imap_measurements,
SUM(flash_measurements*@throttle_ratio<flash_throttled_measurements) flash_throttled_connections, SUM(flash_measurements) flash_measurements, SUM(flash_measurements>0) connections_with_flash_measurements,
count(connection_id) connections_count, SUM(measurements_count) measurements_sum
FROM aggregation_per_connection_glasnost
WHERE measurements_count >= @minmeasurements AND contract_category = 'high'
GROUP BY periphery, isp_id, workingDay, peakHour)";

$queries["periphery_g: drop      "] = "DROP TABLE IF EXISTS aggregation_per_periphery_glasnost"; 
$queries["periphery_g: rename    "] = "RENAME TABLE aggregation_per_periphery_new TO aggregation_per_periphery_glasnost"; 
$queries["periphery_g: index     "] = "CREATE INDEX pindex ON aggregation_per_periphery_glasnost(periphery, workingDay, peakHour, contract)"; 
$queries["periphery_g: analyze   "] = "ANALYZE TABLE aggregation_per_periphery_glasnost"; 



print_r($queries);
$t = microtime(true);
foreach($queries AS $i => $q)
{	
	$qt = microtime(true);
	if(execute_query($q))
		echo "[$i] --> SUCCESS ( time: ".round((microtime(true)-$qt),3)." sec )\n";
	else
		echo "[$i] --> FAIL \n";
}
echo "\n-------------------------------------------------------------------\n";
$t = microtime(true) - $t;
echo "Execution time: ".round($t,3)." sec\n\n";
?>
