/*
 * MLAB Notifier
 * Written by Nikolaos Tsarmpopoulos, for the Hellenic Telecommunications and Post Commission (EETT)
 * Copyright (C) 2011 Hellenic Telecommunications and Post Commission (EETT).
 * Lincensed under the GNU General Public License version 3 (GNU GPLv3) or the European Union Public License (EUPL).
 */

package mlab.notifier;

import java.util.Calendar;
import java.text.SimpleDateFormat;

//import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.util.Random;
import java.io.*;
import java.net.*;

import java.util.Properties;

import javax.jnlp.BasicService;
import javax.jnlp.ServiceManager;
import javax.jnlp.PersistenceService;
import javax.jnlp.UnavailableServiceException;
import javax.jnlp.FileContents;


/**
 *
 * @author ntsarmp
*/

public class MLabNotifier {

    public static boolean pausedScheduler = false;
    public static boolean pausedMeasurements = false;
    public static MenuItem pauseMeasurementsItem;
    public static MenuItem resumeMeasurementsItem;
    public static MenuItem pauseSchedulerItem;
    public static MenuItem resumeSchedulerItem;
    public static MenuItem skipMeasurementItem;

    public static PopupMenu popup;
    public PopupMenu popupMeasurementToolsItem;
    public PopupMenu popupMeasurementFrequencyItem;
    public static TrayIcon trayIcon;

    public int shortPeriodHours = 29;
    public int mediumPeriodHours = 51;
    public int longPeriodHours = 99;

    private PersistenceService ps;
    private BasicService bs;
    private FileContents fc;
    private static ClassLoader cl;

    private static Properties userProperties = new Properties();

    private String measurementToolURL;
    private String personalizedStatsURL="http://broadbandtest.eett.gr/?v=c";
    private final String measurementToolURL_MLAB = "http://www.measurementlab.org/run-ndt?caller=notifier";
    private final String measurementToolURL_SPEBS = "http://broadbandtest.eett.gr/?v=n&l=1&caller=notifier&rl=1";
    private final String measurementToolURL_SPEBS_GR = "http://broadbandtest.eett.gr/?v=n&l=0&caller=notifier&rl=1";


    private boolean reminder = false;
    private boolean redLogo = false;

    public static final String DATE_FORMAT_NOW = "yyyy-MM-dd HH:mm:ss";



    private static ResourceBundle bundle = ResourceBundle.getBundle("mlab/notifier/US_en");



    //
    // main
    //

    public static void main(String[] args) {
        MLabNotifier mLabNotifier = new MLabNotifier();
    }



    //
    // Constructor
    //

    public MLabNotifier(){


        /* Use an appropriate Look and Feel */
        try {

            //UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");

            // Retrieve user settings.

            // Retrieve


        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }

        /* Turn off metal's use of bold fonts */
        //UIManager.put("swing.boldMetal", Boolean.FALSE);
        //Schedule a job for the event-dispatching thread:
        //adding TrayIcon.
        SwingUtilities.invokeLater(new Runnable() {
            
            public void run() {

                userProperties = loadUserProperties();
                
                createAndShowGUI();

                new Thread( new Runnable() {

                    public void run() {

                        while (true){
                            try {

                                if (!pausedScheduler){

                                    if (!reminder){

                                        // Check if this is the time to notify the user to 'Measure Now'.
                                        Calendar calLastMeasurement = Calendar.getInstance();
                                        Calendar calNow = Calendar.getInstance();

                                        if (userProperties.containsKey("user_setting.measurement_tool") && userProperties.containsKey("user_setting.measurements_frequency")){

                                            if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")){
                                                String datetime = userProperties.getProperty("last_spebs_tool_invocation_on");
                                                if (datetime.length()==19) {
                                                    int yearM = new Integer(datetime.substring(0,4));
                                                    int monthM = new Integer(datetime.substring(5,7));
                                                    int dayM = new Integer(datetime.substring(8, 10));
                                                    int hourM = new Integer(datetime.substring(11, 13));
                                                    int minuteM = new Integer(datetime.substring(14, 16));
                                                    int secondM = new Integer(datetime.substring(17, 19));
                                                    calLastMeasurement.set(yearM, monthM-1, dayM, hourM, minuteM, secondM);
                                                } else {
                                                    calLastMeasurement.set(2000, 1, 1, 00, 00, 00);
                                                }
                                            } // if

                                            if (userProperties.getProperty("user_setting.measurement_tool").equals("mlab")){
                                                String datetime = userProperties.getProperty("last_mlab_tool_invocation_on");
                                                if (datetime.length()==19) {
                                                    int yearM = new Integer(datetime.substring(0,4));
                                                    int monthM = new Integer(datetime.substring(5,7));
                                                    int dayM = new Integer(datetime.substring(8, 10));
                                                    int hourM = new Integer(datetime.substring(11, 13));
                                                    int minuteM = new Integer(datetime.substring(14, 16));
                                                    int secondM = new Integer(datetime.substring(17, 19));
                                                    calLastMeasurement.set(yearM, monthM-1, dayM, hourM, minuteM, secondM);
                                                } else {
                                                    calLastMeasurement.set(2000, 1, 1, 00, 00, 00);
                                                }
                                            } // if

                                            long millisecondsM = calLastMeasurement.getTimeInMillis();
                                            long millisecondsN = calNow.getTimeInMillis();

                                            long diff = millisecondsN - millisecondsM;
                                            long diffSeconds = diff / 1000;
                                            long diffMinutes = diff / (60 * 1000);
                                            long diffHours = diff / (60 * 60 * 1000);
                                            long diffDays = diff / (24 * 60 * 60 * 1000);

                                            String frequency = userProperties.getProperty("user_setting.measurements_frequency", "quarterly");

                                            // Every day a measurement is performed, the time would better be different.
                                            int rTime = 12+new Random().nextInt(24);

                                            long diffUserSelected = 0;
                                            if (frequency.equals("daily"))
                                                diffUserSelected = (long) 60 * 22 + rTime; // Every 23 hours (little less than a day).
                                            else if (frequency.equals("weekly"))
                                                diffUserSelected = (long) 60 * 24 * 5 + rTime; // Every 6 days.
                                            else if (frequency.equals("biweekly"))
                                                diffUserSelected = (long) 60 * 24 * 13 + rTime; // Every 14 days.
                                            else if (frequency.equals("monthly"))
                                                diffUserSelected = (long) 60 * 24 * 28 + rTime; // Every 29 days.
                                            else if (frequency.equals("quarterly"))
                                                diffUserSelected = (long) 60 * 24 * 87 + rTime; // Every 88 days.

                                            if (!pausedScheduler && diffMinutes > diffUserSelected){
                                                reminder = true;
                                            } // if


                                        } else {

                                            // If the user has not yet measured his connection, remind him straight away (first use).
                                            if (!reminder){
                                                reminder = true;
                                            }

                                        } // else

                                        long numMillisecondsToSleep = 2000; // 1 min (60.000 milliseconds)
                                        Thread.sleep(numMillisecondsToSleep);

                                    } else {

                                        // If (reminder)

                                        trayIcon.setToolTip("Reminder: Please 'Measure Now' your broadband connection.");
                                        skipMeasurementItem.setEnabled(true);

                                        if (!redLogo){
                                            changeIcon("red");
                                            redLogo = true;
                                        } else {
                                            if (pausedScheduler){
                                                changeIcon("paused");
                                            } else {
                                                changeIcon("active");
                                            } // else
                                            redLogo = false;
                                        } // else

                                        // The delay should be after changeLogo, otherwise the logo does not change to green/orange (thread synchronization).
                                        long numMillisecondsToSleep = 800; // 0.8 sec
                                        Thread.sleep(numMillisecondsToSleep);

                                    } // else


                                } else { // If the user has pause, wait and retry.
                                    long numMillisecondsToSleep = 2000; // 1 min (60.000 milliseconds)
                                    Thread.sleep(numMillisecondsToSleep);

                                }



                            } catch (InterruptedException e) {
                            } // catch

                        } // while
                    } // run
                } ).start();


            }
        });
    }


    private void createAndShowGUI() {

        //Check the SystemTray support
        if (!SystemTray.isSupported()) {
            System.out.println(bundle.getString("error_system_tray_not_supported"));
            return;
        }


        if ( userProperties.containsKey("user_setting.country") && userProperties.containsKey("user_setting.language") ) {
            String countryCode = userProperties.getProperty("user_setting.country");
            String languageCode = userProperties.getProperty("user_setting.language");
            bundle = ResourceBundle.getBundle("mlab/notifier/"+countryCode+"_"+languageCode);

            if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")){
                if ( userProperties.getProperty("user_setting.country").equals("GR") && userProperties.getProperty("user_setting.language").equals("el") )
                    measurementToolURL = measurementToolURL_SPEBS_GR;
                else
                    measurementToolURL = measurementToolURL_SPEBS;
            } // if

        } // if

        popup = new PopupMenu();
        trayIcon = new TrayIcon(createImage("images/mlab_logo_active.png", "tray icon"));
        trayIcon.setImageAutoSize(true);
        trayIcon.setToolTip("M-Lab Notifier");

        final SystemTray tray = SystemTray.getSystemTray();
        
        // Create a popup menu components
        MenuItem aboutItem = new MenuItem(bundle.getString("menu_about"));
        MenuItem measureNowItem = new MenuItem(bundle.getString("menu_measure_now"));
        skipMeasurementItem = new MenuItem(bundle.getString("menu_skip_measurement"));
        popupMeasurementToolsItem = new PopupMenu(bundle.getString("menu_measurement_tools"));
        popupMeasurementFrequencyItem = new PopupMenu(bundle.getString("menu_measurement_frequency"));
        pauseMeasurementsItem = new MenuItem(bundle.getString("menu_pause_measurements"));
        resumeMeasurementsItem = new MenuItem(bundle.getString("menu_resume_measurements"));
        pauseSchedulerItem = new MenuItem(bundle.getString("menu_pause"));
        resumeSchedulerItem = new MenuItem(bundle.getString("menu_resume"));
        MenuItem personalizedStatsItem = new MenuItem(bundle.getString("menu_personalized_stats"));
        //CheckboxMenuItem cb1 = new CheckboxMenuItem("Pause");
        Menu languageMenu = new Menu(bundle.getString("menu_language"));
        MenuItem languageEnglishItem = new MenuItem("English");
        MenuItem languageGreekItem = new MenuItem("Ελληνικά");
        MenuItem exitItem = new MenuItem(bundle.getString("menu_exit"));


        // Add components to the popupMeasurementTools
        final MenuItem mlabItem = new MenuItem(bundle.getString("menu_tool_mlab"));
        final MenuItem spebsItem = new MenuItem(bundle.getString("menu_tool_spebs"));


        if (userProperties.containsKey("user_setting.measurement_tool")){

            if (userProperties.getProperty("user_setting.measurement_tool").equals("mlab")){
                mlabItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default value
            } // if

            if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")){
                spebsItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

        } // if

        popupMeasurementToolsItem.add(mlabItem);
        popupMeasurementToolsItem.add(spebsItem);

        final MenuItem everyDayItem = new MenuItem(bundle.getString("menu_frequency_daily"));
        final MenuItem everyWeekItem = new MenuItem(bundle.getString("menu_frequency_weekly"));
        final MenuItem everyOtherWeekItem = new MenuItem(bundle.getString("menu_frequency_biweekly"));
        final MenuItem everyMonthItem = new MenuItem(bundle.getString("menu_frequency_monthly"));
        final MenuItem everyQuarterItem = new MenuItem(bundle.getString("menu_frequency_quarterly"));

        if (userProperties.containsKey("user_setting.measurements_frequency")){

            if (userProperties.getProperty("user_setting.measurements_frequency").equals("daily")){
                everyDayItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

            if (userProperties.getProperty("user_setting.measurements_frequency").equals("weekly")){
                everyWeekItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

            if (userProperties.getProperty("user_setting.measurements_frequency").equals("biweekly")){
                everyOtherWeekItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

            if (userProperties.getProperty("user_setting.measurements_frequency").equals("monthly")){
                everyMonthItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

            if (userProperties.getProperty("user_setting.measurements_frequency").equals("quarterly")){
                everyQuarterItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

        } // if


        popupMeasurementFrequencyItem.add(everyDayItem);
        popupMeasurementFrequencyItem.add(everyWeekItem);
        popupMeasurementFrequencyItem.add(everyOtherWeekItem);
        popupMeasurementFrequencyItem.add(everyMonthItem);
        popupMeasurementFrequencyItem.add(everyQuarterItem);

        //Add components to popup menu
        popup.add(measureNowItem);

        // Depending on the status of the Notifier, the option could be disabled.
        if (!reminder)
            skipMeasurementItem.setEnabled(false);
        popup.add(skipMeasurementItem);

        
        //popup.addSeparator();
        //popup.add(personalizedStatsItem);

        popup.addSeparator();
        popup.add(popupMeasurementToolsItem);
        popup.add(popupMeasurementFrequencyItem);
        //popup.add(cb1);
        //popup.addSeparator();
        popup.add(languageMenu);
        
        languageMenu.add(languageEnglishItem);
        languageMenu.add(languageGreekItem);


        if ( userProperties.containsKey("user_setting.country") && userProperties.containsKey("user_setting.language") ) {

            if (userProperties.getProperty("user_setting.country").equals("GR")
                    && userProperties.getProperty("user_setting.language").equals("el") ){
                languageGreekItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

            if (userProperties.getProperty("user_setting.country").equals("US")
                    && userProperties.getProperty("user_setting.language").equals("en") ){
                languageEnglishItem.setFont(new Font("Arial", java.awt.Font.BOLD, 12)); // Default Value
            } // if

        } // if


        popup.addSeparator();
        popup.add(pauseSchedulerItem);
        popup.addSeparator();
        popup.add(aboutItem);
        popup.addSeparator();
        //popup.add(pauseMeasurementsItem);
        popup.add(exitItem);
        
        trayIcon.setPopupMenu(popup);
        
        try {
            tray.add(trayIcon);
        } catch (AWTException e) {
            System.out.println(bundle.getString("error_tray_icon_could_not_be_added"));
            return;
        }
        
        trayIcon.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                //                JOptionPane.showMessageDialog(null,
                //                        "This dialog box is run from System Tray");
            } // actionPerformed
        }); // ActionListener

        aboutItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                URL imageURL = MLabNotifier.class.getResource("images/mlab_scheduler_logo.png");
                String lastMLABToolInvokedOn = userProperties.getProperty("last_mlab_tool_invocation_on");
                String lastSPEBSToolInvokedOn = userProperties.getProperty("last_spebs_tool_invocation_on");
                String dialogBoxText = bundle.getString("about_version") +"\n"+
                        bundle.getString("about_copyright")+"\n\n"+
                        bundle.getString("about_developed_by") +"\n"+
                        bundle.getString("about_ntua") +"\n\n"+
                        bundle.getString("about_new_america_foundation") +"\n"+
                        bundle.getString("about_new_america_foundation_2") +"\n"+
                        bundle.getString("about_new_america_foundation_3") +"\n\n"+
                        bundle.getString("about_mlab_last_invoked")+" "+(lastMLABToolInvokedOn.length()>0?lastMLABToolInvokedOn:bundle.getString("about_never_invoked"))+"\n"+
                        bundle.getString("about_spebs_last_invoked")+" "+(lastSPEBSToolInvokedOn.length()>0?lastSPEBSToolInvokedOn:bundle.getString("about_never_invoked"));

                JOptionPane.showMessageDialog((Component)null, dialogBoxText, bundle.getString("about_title"),
                        javax.swing.JOptionPane.INFORMATION_MESSAGE, new ImageIcon(imageURL));
            } // actionPerformed
        }); // ActionListener

        personalizedStatsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")) {
                    BareBonesBrowserLaunch.openURL(personalizedStatsURL);
                } // if
            } // actionPerformed
        }); // ActionListener

        
        
        //
        // LANGUAGE ENGLISH
        //
        languageEnglishItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.country", "US");
                userProperties.setProperty("user_setting.language", "en");
                bundle = ResourceBundle.getBundle("mlab/notifier/US_en");
                tray.remove(trayIcon);
                createAndShowGUI();
            } // actionPerformed
        }); // ActionListener


        //
        // LANGUAGE GREEK
        //
        languageGreekItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.country", "GR");
                userProperties.setProperty("user_setting.language", "el");
                bundle = ResourceBundle.getBundle("mlab/notifier/GR_el");
                tray.remove(trayIcon);
                createAndShowGUI();
            } // actionPerformed
        }); // ActionListener



        //
        // Handle change of Measurement Frequency
        //
        everyDayItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurements_frequency", "daily");
                ((MenuItem)everyDayItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)everyWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyOtherWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyMonthItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyQuarterItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener

        everyWeekItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurements_frequency", "weekly");
                ((MenuItem)everyDayItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyWeekItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)everyOtherWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyMonthItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyQuarterItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener

        everyOtherWeekItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurements_frequency", "biweekly");
                ((MenuItem)everyDayItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyOtherWeekItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)everyMonthItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyQuarterItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener

        everyMonthItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurements_frequency", "monthly");
                ((MenuItem)everyDayItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyOtherWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyMonthItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)everyQuarterItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener

        everyQuarterItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurements_frequency", "quarterly");
                ((MenuItem)everyDayItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyOtherWeekItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyMonthItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                ((MenuItem)everyQuarterItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener




        //
        // Handle change of Measurement Tool
        //
        mlabItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurement_tool", "mlab");
                measurementToolURL = measurementToolURL_MLAB;
                ((MenuItem)mlabItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)spebsItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties
            } // actionPerformed
        }); // ActionListener

        spebsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                userProperties.setProperty("user_setting.measurement_tool", "spebs");

                if ( userProperties.getProperty("user_setting.country").equals("GR") && userProperties.getProperty("user_setting.language").equals("el") )
                    measurementToolURL = measurementToolURL_SPEBS_GR;
                else
                    measurementToolURL = measurementToolURL_SPEBS;

                ((MenuItem)spebsItem).setFont(new Font("Arial", java.awt.Font.BOLD, 12));
                ((MenuItem)mlabItem).setFont(new Font("Arial", java.awt.Font.PLAIN, 12));
                // TO-DO: Save userProperties

            } // actionPerformed
        }); // ActionListener




        //
        // Handle Measure Now.
        //
        measureNowItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                BareBonesBrowserLaunch.openURL(measurementToolURL);

                if (userProperties.containsKey("user_setting.measurement_tool")){

                    if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")){
                        userProperties.setProperty("last_spebs_tool_invocation_on", now());
                    } // if

                    if (userProperties.getProperty("user_setting.measurement_tool").equals("mlab")){
                        userProperties.setProperty("last_mlab_tool_invocation_on", now());
                    } // if

                } // if

                if (reminder == true){

                    trayIcon.setToolTip(bundle.getString("tooltip_mlab_notifier"));

                    reminder = false;
                    redLogo = false;

                    if (pausedScheduler){
                        changeIcon("paused");
                    } else {
                        changeIcon("active");
                    } // else


                }
                
            } // actionPerformed

        }); // ActionListener



        //
        // Handle Skip Measurement
        //
        skipMeasurementItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {


                if (reminder == true){

                    if (userProperties.containsKey("user_setting.measurement_tool")){

                        if (userProperties.getProperty("user_setting.measurement_tool").equals("spebs")){
                            userProperties.setProperty("last_spebs_tool_invocation_on", now());
                        } // if

                        if (userProperties.getProperty("user_setting.measurement_tool").equals("mlab")){
                            userProperties.setProperty("last_mlab_tool_invocation_on", now());
                        } // if

                    } // if


                    trayIcon.setToolTip(bundle.getString("tooltip_mlab_notifier"));
                    
                    skipMeasurementItem.setEnabled(false);

                    reminder = false;
                    redLogo = false;

                    if (pausedScheduler){
                        changeIcon("paused");
                    } else {
                        changeIcon("active");
                    } // else


                }

            } // actionPerformed

        }); // ActionListener



        //
        // Handle Pause Measurements
        //
        pauseMeasurementsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pausedMeasurements=true;
                popup.remove(pauseMeasurementsItem);
                popup.insert(resumeMeasurementsItem, 7);

            } // actionPerformed
        }); // ActionListener

        pauseSchedulerItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pausedScheduler = true;
                popup.remove(pauseSchedulerItem);
                popup.insert(resumeSchedulerItem, 7);
                trayIcon.setImage(createImage("images/mlab_logo_paused.png", "tray icon"));
            } // actionPerformed
        }); // ActionListener

        resumeMeasurementsItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pausedMeasurements=false;
                popup.remove (resumeMeasurementsItem);
                popup.insert(pauseMeasurementsItem, 7);

            } // actionPerformed
        }); // ActionListener

       resumeSchedulerItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pausedScheduler = false;
                popup.remove (resumeSchedulerItem);
                popup.insert(pauseSchedulerItem, 7);
                trayIcon.setImage(createImage("images/mlab_logo_active.png", "tray icon"));
            } // actionPerformed
        }); // ActionListener

        /*
        cb1.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                int cb1Id = e.getStateChange();
                if (cb1Id == ItemEvent.SELECTED){
                    //trayIcon.setImageAutoSize(true);
                } else {
                    trayIcon.setImageAutoSize(false);
                }
            }
        });
        */
        
        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveUserProperties(userProperties);
                tray.remove(trayIcon);
                System.exit(0);
            }
        });


    } // createAndShowGUI

    
    //
    // Returns the current date and time in mysql format
    //
    public static String now() {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
        return sdf.format(cal.getTime());
    }


    //
    // Changes the application icon on the toolbar.
    //
    synchronized private void changeIcon(String state) {

        if (state.equals("active"))
            trayIcon.setImage(createImage("images/mlab_logo_active.png", "tray icon"));
        else if (state.equals("paused"))
            trayIcon.setImage(createImage("images/mlab_logo_paused.png", "tray icon"));
        else if (state.equals("red"))
            trayIcon.setImage(createImage("images/mlab_logo_red_all.png", "tray icon"));
    
    } // changeLogo

    
    protected Image createImage(String status){
        
        int width = 32;
        int height = 32;
        BufferedImage bufImage = new BufferedImage(width, height,BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufImage.getGraphics();
        g.setColor(Color.blue);
        g.setFont(new Font("Arial", java.awt.Font.PLAIN, 20));
        g.drawString(status, 10, 25);

        Image i = bufImage.getScaledInstance(32, 32, java.awt.Image.SCALE_SMOOTH);
        ImageIcon icon = new ImageIcon(bufImage);
        
        return new ImageIcon(i).getImage();
    
    } // createImage


    //Obtain the image URL
    protected Image createImage(String path, String description) {
        /* return createImage("1"); */
        
        URL imageURL = MLabNotifier.class.getResource(path);
        
        if (imageURL == null) {

            System.err.println(bundle.getString("error_resource_not_found") + path);
            return null;

        } else {

            return (new ImageIcon(imageURL, description)).getImage();
        
        } // else
        
    } // createImage



     private Properties loadUserProperties(){

         Properties p = new Properties();

         try {

             ps = (PersistenceService)ServiceManager.lookup("javax.jnlp.PersistenceService");
             bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");

         } catch (UnavailableServiceException e) {

             ps = null;
             bs = null;

         } // catch

        if (ps != null && bs != null) {

            try {

                URL codebase = bs.getCodeBase();
                URL url = new URL(codebase.toString() + "perstest");

                fc = ps.get(url);

                if (fc!=null){

                    if (fc.canRead()){
                        InputStream os = fc.getInputStream();
                        p.load(os);

                    } // if

                } // if



            } catch (Exception e) {

                e.printStackTrace();

            } // catch

        } // if

        String systemLanguage = System.getProperty("user.language");
        String systemCountry = System.getProperty("user.country");

        // Set Default Keys and Values
        if (!p.containsKey("user_setting.country")) {
            if (systemCountry!=null && systemCountry.equals("GR"))
                p.setProperty("user_setting.country", "GR");
            else
                p.setProperty("user_setting.country", "US");
        } // if
        if (!p.containsKey("user_setting.language")) {
            if (systemLanguage!=null && systemLanguage.equals("el"))
                p.setProperty("user_setting.language", "el");
            else
                p.setProperty("user_setting.language", "en");
        } // if
        if (!p.containsKey("user_setting.measurements_frequency")) {
            p.setProperty("user_setting.measurements_frequency", "weekly");
        } // if
        if (!p.containsKey("user_setting.measurement_tool")) {
            p.setProperty("user_setting.measurement_tool", "spebs");
        } // if

        if (!p.containsKey("last_mlab_tool_invocation_on")) {
            p.setProperty("last_mlab_tool_invocation_on", "");
        } // if

        if (!p.containsKey("last_spebs_tool_invocation_on")) {
            p.setProperty("last_spebs_tool_invocation_on", "");
        } // if


        if (p.getProperty("user_setting.measurement_tool").equals("mlab")){
            measurementToolURL = measurementToolURL_MLAB;
        }

        if (p.getProperty("user_setting.measurement_tool").equals("spebs")){
            if ( p.getProperty("user_setting.country").equals("GR") && p.getProperty("user_setting.language").equals("el") )
                measurementToolURL = measurementToolURL_SPEBS_GR;
            else
                measurementToolURL = measurementToolURL_SPEBS;
        } // if


         return p;
     }

     
     private void saveUserProperties(Properties p){

         if (p==null)
             return;

         try {

             ps = (PersistenceService)ServiceManager.lookup("javax.jnlp.PersistenceService");
             bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");

         } catch (UnavailableServiceException e) {

             ps = null;
             bs = null;

         } // catch

        if (ps != null && bs != null) {

            try {

                URL codebase = bs.getCodeBase();
                URL url = new URL(codebase.toString() + "perstest");

                // If it already exists, it will move on...
                try {

                    ps.create(url, 4096);

                } catch (Exception e){
                }

                fc = ps.get(url);


                if (fc.canWrite()){
                    OutputStream os = fc.getOutputStream(true);
                    p.store(os, null);

                } // if


            } catch (Exception e) {

                System.err.println(e.getMessage());

            } // catch

        } // if
         

     } // saveUserProperties


} // SPEBSScheduler

